@@ -40,25 +40,24 @@
 #include <config.h>
 
 #include <arpa/inet.h>
-#include <err.h>
 #include <errno.h>
 #include <inttypes.h>
 #include <langinfo.h>
 #include <math.h>
 #include <netinet/in.h>
-#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <time.h>
 
 #include "close-stream.h"
-#include "dhcpd-pools.h"
+#include "error.h"
 #include "progname.h"
 #include "strftime.h"
 
-/*! \brief Text output format, which is the default.
- * FIXME: This function should return void. */
+#include "dhcpd-pools.h"
+
+/*! \brief Text output format, which is the default. */
 int output_txt(void)
 {
 	unsigned int i;
@@ -67,12 +66,12 @@ int output_txt(void)
 	struct shared_network_t *shared_p;
 	int ret;
 	FILE *outfile;
-	int max_ipaddr_length = config.dhcp_version == VERSION_6 ? 39 : 16;
+	int max_ipaddr_length = config.ip_version == IPv6 ? 39 : 16;
 
 	if (config.output_file[0]) {
 		outfile = fopen(config.output_file, "w+");
 		if (outfile == NULL) {
-			err(EXIT_FAILURE, "output_txt: %s", config.output_file);
+			error(EXIT_FAILURE, errno, "output_txt: %s", config.output_file);
 		}
 	} else {
 		outfile = stdout;
@@ -82,7 +81,7 @@ int output_txt(void)
 	range_size = get_range_size(range_p);
 	shared_p = shared_networks;
 
-	if (config.output_limit[0] & BIT1) {
+	if (config.header_limit & R_BIT) {
 		fprintf(outfile, "Ranges:\n");
 		fprintf
 		    (outfile,
@@ -92,12 +91,12 @@ int output_txt(void)
 		     "first ip",
 		     max_ipaddr_length,
 		     "last ip", "max", "cur", "percent", "touch", "t+c", "t+c perc");
-		if (config.backups_found == true) {
+		if (config.backups_found == 1) {
 			fprintf(outfile, "     bu  bu perc");
 		}
 		fprintf(outfile, "\n");
 	}
-	if (config.output_limit[1] & BIT1) {
+	if (config.number_limit & R_BIT) {
 		for (i = 0; i < num_ranges; i++) {
 			if (range_p->shared_net) {
 				fprintf(outfile, "%-20s", range_p->shared_net->name);
@@ -119,7 +118,7 @@ int output_txt(void)
 				range_p->touched,
 				range_p->touched + range_p->count,
 				(float)(100 * (range_p->touched + range_p->count)) / range_size);
-			if (config.backups_found == true) {
+			if (config.backups_found == 1) {
 				fprintf(outfile, "%7g %8.3f",
 					range_p->backups,
 					(float)(100 * range_p->backups) / range_size);
@@ -129,19 +128,19 @@ int output_txt(void)
 			range_size = get_range_size(range_p);
 		}
 	}
-	if (config.output_limit[1] & BIT1 && config.output_limit[0] & BIT2) {
+	if (config.number_limit & R_BIT && config.header_limit & S_BIT) {
 		fprintf(outfile, "\n");
 	}
-	if (config.output_limit[0] & BIT2) {
+	if (config.header_limit & S_BIT) {
 		fprintf(outfile, "Shared networks:\n");
 		fprintf(outfile,
 			"name                   max   cur     percent  touch    t+c  t+c perc");
-		if (config.backups_found == true) {
+		if (config.backups_found == 1) {
 			fprintf(outfile, "     bu  bu perc");
 		}
 		fprintf(outfile, "\n");
 	}
-	if (config.output_limit[1] & BIT2) {
+	if (config.number_limit & S_BIT) {
 		for (i = 0; i < num_shared_networks; i++) {
 			shared_p++;
 			fprintf(outfile,
@@ -149,12 +148,12 @@ int output_txt(void)
 				shared_p->name, shared_p->available,
 				shared_p->used,
 				shared_p->available ==
-				0 ? -NAN : (float)(100 * shared_p->used) / shared_p->available,
+				0 ? NAN : (float)(100 * shared_p->used) / shared_p->available,
 				shared_p->touched, shared_p->touched + shared_p->used,
 				shared_p->available ==
-				0 ? -NAN : ((float)(100 * (shared_p->touched + shared_p->used)) /
+				0 ? NAN : ((float)(100 * (shared_p->touched + shared_p->used)) /
 					    shared_p->available));
-			if (config.backups_found == true) {
+			if (config.backups_found == 1) {
 				fprintf(outfile, "%7g %8.3f",
 					shared_p->backups,
 					(float)(100 * shared_p->backups) / shared_p->available);
@@ -163,36 +162,36 @@ int output_txt(void)
 			fprintf(outfile, "\n");
 		}
 	}
-	if (config.output_limit[1] & BIT2 && config.output_limit[0] & BIT3) {
+	if (config.number_limit & S_BIT && config.header_limit & A_BIT) {
 		fprintf(outfile, "\n");
 	}
-	if (config.output_limit[0] & BIT3) {
+	if (config.header_limit & A_BIT) {
 		fprintf(outfile, "Sum of all ranges:\n");
 		fprintf(outfile,
 			"name                   max   cur     percent  touch    t+c  t+c perc");
 
-		if (config.backups_found == true) {
+		if (config.backups_found == 1) {
 			fprintf(outfile, "     bu  bu perc");
 		}
 		fprintf(outfile, "\n");
 	}
-	if (config.output_limit[1] & BIT3) {
+	if (config.number_limit & A_BIT) {
 		fprintf(outfile, "%-20s %5g %5g %10.3f %7g %6g %9.3f",
 			shared_networks->name,
 			shared_networks->available,
 			shared_networks->used,
 			shared_networks->available ==
-			0 ? -NAN : (float)(100 * shared_networks->used) /
+			0 ? NAN : (float)(100 * shared_networks->used) /
 			shared_networks->available, shared_networks->touched,
 			shared_networks->touched + shared_networks->used,
 			shared_networks->available ==
-			0 ? -NAN : (float)(100 *
+			0 ? NAN : (float)(100 *
 					   (shared_networks->touched +
 					    shared_networks->used)) / shared_networks->available);
 
-		if (config.backups_found == true) {
+		if (config.backups_found == 1) {
 			fprintf(outfile, "%7g %8.3f",
-				shared_networks->available == 0 ? -NAN : shared_networks->backups,
+				shared_networks->available == 0 ? NAN : shared_networks->backups,
 				(float)(100 * shared_networks->backups) /
 				shared_networks->available);
 		}
@@ -201,21 +200,19 @@ int output_txt(void)
 	if (outfile == stdout) {
 		ret = fflush(stdout);
 		if (ret) {
-			warn("output_txt: fflush");
+			error(EXIT_FAILURE, errno, "output_txt: fflush");
 		}
 	} else {
 		ret = close_stream(outfile);
 		if (ret) {
-			warn("output_txt: fclose");
+			error(EXIT_FAILURE, errno, "output_txt: fclose");
 		}
 	}
 
 	return 0;
 }
 
-/*! \brief The xml output formats.
- * FIXME: This function should return void.
- */
+/*! \brief The xml output formats. */
 int output_xml(void)
 {
 	unsigned int i;
@@ -228,7 +225,7 @@ int output_xml(void)
 	if (config.output_file[0]) {
 		outfile = fopen(config.output_file, "w+");
 		if (outfile == NULL) {
-			err(EXIT_FAILURE, "output_xml: %s", config.output_file);
+			error(EXIT_FAILURE, errno, "output_xml: %s", config.output_file);
 		}
 	} else {
 		outfile = stdout;
@@ -240,8 +237,9 @@ int output_xml(void)
 
 	fprintf(outfile, "<dhcpstatus>\n");
 
-	if (config.output_format[0] == 'X' || config.output_format[0] == 'J') {
+	if (config.print_mac_addreses == 1) {
 		struct leases_t *l;
+
 		for (l = leases; l != NULL; l = l->hh.next) {
 			if (l->type == ACTIVE) {
 				fputs("<active_lease>\n\t<ip>", outfile);
@@ -255,7 +253,7 @@ int output_xml(void)
 		}
 	}
 
-	if (config.output_limit[1] & BIT1) {
+	if (config.number_limit & R_BIT) {
 		for (i = 0; i < num_ranges; i++) {
 			fprintf(outfile, "<subnet>\n");
 			if (range_p->shared_net) {
@@ -276,7 +274,7 @@ int output_xml(void)
 		}
 	}
 
-	if (config.output_limit[1] & BIT2) {
+	if (config.number_limit & S_BIT) {
 		for (i = 0; i < num_shared_networks; i++) {
 			shared_p++;
 			fprintf(outfile, "<shared-network>\n");
@@ -290,7 +288,7 @@ int output_xml(void)
 		}
 	}
 
-	if (config.output_limit[0] & BIT3) {
+	if (config.header_limit & A_BIT) {
 		fprintf(outfile, "<summary>\n");
 		fprintf(outfile, "\t<location>%s</location>\n", shared_networks->name);
 		fprintf(outfile, "\t<defined>%g</defined>\n", shared_networks->available);
@@ -305,21 +303,19 @@ int output_xml(void)
 	if (outfile == stdout) {
 		ret = fflush(stdout);
 		if (ret) {
-			warn("output_xml: fflush");
+			error(EXIT_FAILURE, errno, "output_xml: fflush");
 		}
 	} else {
 		ret = close_stream(outfile);
 		if (ret) {
-			warn("output_xml: fclose");
+			error(EXIT_FAILURE, errno, "output_xml: fclose");
 		}
 	}
 
 	return 0;
 }
 
-/*! \brief The json output formats.
- * FIXME: This function should return void.
- */
+/*! \brief The json output formats. */
 int output_json(void)
 {
 	unsigned int i = 0;
@@ -333,7 +329,7 @@ int output_json(void)
 	if (config.output_file[0]) {
 		outfile = fopen(config.output_file, "w+");
 		if (outfile == NULL) {
-			err(EXIT_FAILURE, "output_json: %s", config.output_file);
+			error(EXIT_FAILURE, errno, "output_json: %s", config.output_file);
 		}
 	} else {
 		outfile = stdout;
@@ -346,8 +342,9 @@ int output_json(void)
 
 	fprintf(outfile, "{\n");
 
-	if (config.output_format[0] == 'X' || config.output_format[0] == 'J') {
+	if (config.print_mac_addreses == 1) {
 		struct leases_t *l;
+
 		fprintf(outfile, "   \"active_leases\": [");
 		for (l = leases; l != NULL; l = l->hh.next) {
 			if (l->type == ACTIVE) {
@@ -369,7 +366,7 @@ int output_json(void)
 		sep++;
 	}
 
-	if (config.output_limit[1] & BIT1) {
+	if (config.number_limit & R_BIT) {
 		if (sep) {
 			fprintf(outfile, ",\n");
 		}
@@ -401,7 +398,7 @@ int output_json(void)
 		sep++;
 	}
 
-	if (config.output_limit[1] & BIT2) {
+	if (config.number_limit & S_BIT) {
 		if (sep) {
 			fprintf(outfile, ",\n");
 		}
@@ -424,7 +421,7 @@ int output_json(void)
 		sep++;
 	}
 
-	if (config.output_limit[0] & BIT3) {
+	if (config.header_limit & A_BIT) {
 		if (sep) {
 			fprintf(outfile, ",\n");
 		}
@@ -443,12 +440,12 @@ int output_json(void)
 	if (outfile == stdout) {
 		ret = fflush(stdout);
 		if (ret) {
-			warn("output_json: fflush");
+			error(EXIT_FAILURE, errno, "output_json: fflush");
 		}
 	} else {
 		ret = close_stream(outfile);
 		if (ret) {
-			warn("output_json: fclose");
+			error(EXIT_FAILURE, errno, "output_json: fclose");
 		}
 	}
 
@@ -465,72 +462,33 @@ static void html_header(FILE *restrict f)
 	struct tm *tmp, result;
 
 	struct stat statbuf;
+
 	stat(config.dhcpdlease_file, &statbuf);
 
 	tmp = localtime_r(&statbuf.st_mtime, &result);
 	if (tmp == NULL) {
-		err(EXIT_FAILURE, "html_header: localtime");
+		error(EXIT_FAILURE, errno, "html_header: localtime");
 	}
 	if (strftime(outstr, sizeof(outstr), nl_langinfo(D_T_FMT), &result) == 0) {
-		errx(EXIT_FAILURE, "html_header: strftime returned 0");
+		error(EXIT_FAILURE, 0, "html_header: strftime returned 0");
 	}
-
-	fprintf(f, "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"\n");
-	fprintf(f, "\"http://www.w3.org/TR/html4/loose.dtd\">\n");
+	fprintf(f, "<!DOCTYPE html>\n");
 	fprintf(f, "<html>\n");
 	fprintf(f, "<head>\n");
-	fprintf(f, "<meta http-equiv=\"Content-Type\" ");
-	fprintf(f, "content=\"text/html; charset=iso-8859-1\">\n");
-	fprintf(f, "<title>ISC dhcpd stats</title>\n");
+	fprintf(f, "<title>ISC dhcpd dhcpd-pools output</title>\n");
+	fprintf(f, "<meta charset=\"utf-8\">\n");
+	fprintf(f, "<meta http-equiv=\"X-UA-Compatible\" content=\"IE=edge\">\n");
+	fprintf(f, "<meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">\n");
+	fprintf(f, "<link rel=\"stylesheet\" href=\"https://maxcdn.bootstrapcdn.com/bootstrap/3.3.4/css/bootstrap.min.css\" type=\"text/css\">\n");
+	fprintf(f, "<link rel=\"stylesheet\" href=\"https://cdn.datatables.net/1.10.10/css/jquery.dataTables.min.css\" type=\"text/css\">");
 	fprintf(f, "<style type=\"text/css\">\n");
-	fprintf(f, "<!--\n");
-	fprintf(f, "table.dhcpd-pools {\n");
-	fprintf(f, "color: black;\n");
-	fprintf(f, "vertical-align: middle;\n");
-	fprintf(f, "text-align: center;\n");
-	fprintf(f, "}\n");
-	fprintf(f, "table.dhcpd-pools th.section {\n");
-	fprintf(f, "color: black;\n");
-	fprintf(f, "font-size: large;\n");
-	fprintf(f, "vertical-align: middle;\n");
-	fprintf(f, "text-align: left;\n");
-	fprintf(f, "}\n");
-	fprintf(f, "table.dhcpd-pools th.lalign {\n");
-	fprintf(f, "color: black;\n");
-	fprintf(f, "vertical-align: middle;\n");
-	fprintf(f, "text-align: left;\n");
-	fprintf(f, "text-decoration: underline;\n");
-	fprintf(f, "}\n");
-	fprintf(f, "table.dhcpd-pools th.ralign {\n");
-	fprintf(f, "color: black;\n");
-	fprintf(f, "vertical-align: middle;\n");
-	fprintf(f, "text-align: right;\n");
-	fprintf(f, "text-decoration: underline;\n");
-	fprintf(f, "}\n");
-	fprintf(f, "table.dhcpd-pools td.lalign {\n");
-	fprintf(f, "color: black;\n");
-	fprintf(f, "vertical-align: middle;\n");
-	fprintf(f, "text-align: left;\n");
-	fprintf(f, "}\n");
-	fprintf(f, "table.dhcpd-pools td.ralign {\n");
-	fprintf(f, "color: black;\n");
-	fprintf(f, "vertical-align: middle;\n");
-	fprintf(f, "text-align: right;\n");
-	fprintf(f, "}\n");
-	fprintf(f, "p.created {\n");
-	fprintf(f, "font-size: small;\n");
-	fprintf(f, "color: grey;\n");
-	fprintf(f, "}\n");
-	fprintf(f, "p.updated {\n");
-	fprintf(f, "font-size: small;\n");
-	fprintf(f, "color: grey;\n");
-	fprintf(f, "font-style: italic;\n");
-	fprintf(f, "}\n");
-	fprintf(f, "-->\n");
+	fprintf(f, "table.dhcpd-pools th { text-transform: capitalize }\n");
 	fprintf(f, "</style>\n");
 	fprintf(f, "</head>\n");
 	fprintf(f, "<body>\n");
-	fprintf(f, "<a name=\"ranges\">The lease file mtime: %s</a>", outstr);
+	fprintf(f, "<div class=\"container\">\n");
+	fprintf(f, "<h2>ISC DHCPD status</h2>\n");
+	fprintf(f, "<small>File %s was last modified at %s</small><hr />\n", config.dhcpdlease_file, outstr);
 }
 
 /*! \brief Footer for full html output format.
@@ -539,36 +497,36 @@ static void html_header(FILE *restrict f)
  */
 static void html_footer(FILE *restrict f)
 {
-	fprintf(f, "<p><br></p>\n");
-	fprintf(f, "<hr>\n");
-	fprintf(f, "<p class=\"created\">\nData generated by ");
-	fprintf(f, "<a href=\"%s\">", PACKAGE_URL);
-	fprintf(f, "%s</a>.\n</p>\n", PACKAGE_STRING);
-	fprintf(f, "<p class=\"updated\">\n");
-	fprintf(f, "<script type=\"text/javascript\">\n");
-	fprintf(f, "document.write(\"Last Updated On \" + ");
-	fprintf(f, "document.lastModified + \".\")\n");
-	fprintf(f, "</script>\n<br>\n</p>\n");
-	fprintf(f, "</body>\n");
-	fprintf(f, "</html>\n");
+	fprintf(f, "<br /><div class=\"well well-lg\">\n");
+	fprintf(f, "<small>Generated using %s<br />\n", PACKAGE_STRING);
+	fprintf(f, "More info at <a href=\"%s\">%s</a>\n", PACKAGE_URL, PACKAGE_URL);
+	fprintf(f, "</small></div></div>\n");
+	fprintf(f, "<script src=\"https://code.jquery.com/jquery-2.1.4.min.js\" type=\"text/javascript\"></script>\n");
+	fprintf(f, "<script src=\"https://maxcdn.bootstrapcdn.com/bootstrap/3.3.4/js/bootstrap.min.js\" type=\"text/javascript\"></script>\n");
+	fprintf(f, "<script src=\"https://cdn.datatables.net/1.10.10/js/jquery.dataTables.min.js\" type=\"text/javascript\"></script>\n");
+	fprintf(f, "<script type=\"text/javascript\" class=\"init\">$(document).ready(function() { $('#s').DataTable(); } );</script>\n");
+	fprintf(f, "<script type=\"text/javascript\" class=\"init\">$(document).ready(function() { $('#r').DataTable(); } );</script>\n");
+	fprintf(f, "</body></html>\n");
 }
 
-/*! \brief A new row for html output format.
+/*! \brief Start a html tag.
  *
  * \param f Output file descriptor.
+ * \param tag The html tag.
  */
-static void newrow(FILE *restrict f)
+static void start_tag(FILE *restrict f, char const *restrict tag)
 {
-	fprintf(f, "<tr>\n");
+	fprintf(f, "<%s>\n", tag);
 }
 
-/*! \brief End a row for html output format.
+/*! \brief End a html tag.
  *
  * \param f Output file descriptor.
+ * \param tag The html tag.
  */
-static void endrow(FILE *restrict f)
+static void end_tag(FILE *restrict f, char const *restrict tag)
 {
-	fprintf(f, "</tr>\n");
+	fprintf(f, "</%s>\n", tag);
 }
 
 /*! \brief Line with text in html output format.
@@ -579,9 +537,9 @@ static void endrow(FILE *restrict f)
  * \param text Actual payload of the printout.
  */
 static void output_line(FILE *restrict f, char const *restrict type,
-			char const *restrict class, char const *restrict text)
+			char const *restrict text)
 {
-	fprintf(f, "<%s class=\"%s\">%s</%s>\n", type, class, text, type);
+	fprintf(f, "<%s>%s</%s>\n", type, text, type);
 }
 
 /*! \brief Line with digit in html output format.
@@ -592,7 +550,7 @@ static void output_line(FILE *restrict f, char const *restrict type,
  */
 static void output_double(FILE *restrict f, char const *restrict type, double d)
 {
-	fprintf(f, "<%s class=\"ralign\">%g</%s>\n", type, d, type);
+	fprintf(f, "<%s>%g</%s>\n", type, d, type);
 }
 
 /*! \brief Line with float in html output format.
@@ -601,20 +559,18 @@ static void output_double(FILE *restrict f, char const *restrict type, double d)
  * \param type HTML tag name.
  * \param fl Actual payload of the printout.
  */
-static void output_float(FILE *f, char const *restrict type, float fl)
+static void output_float(FILE *restrict f, char const *restrict type, float fl)
 {
-	fprintf(f, "<%s class=\"ralign\">%.3f</%s>\n", type, fl, type);
+	fprintf(f, "<%s>%.3f</%s>\n", type, fl, type);
 }
 
 /*! \brief Begin table in html output format.
  *
  * \param f Output file descriptor.
  */
-static void table_start(FILE *restrict f)
+static void table_start(FILE *restrict f, char const *restrict id, char const *restrict summary)
 {
-	fprintf(f, "<table width=\"75%%\" ");
-	fprintf(f, "class=\"%s\" ", PACKAGE_NAME);
-	fprintf(f, "summary=\"ISC dhcpd pool usage report\">\n");
+	fprintf(f, "<table id=\"%s\" class=\"dhcpd-pools order-column table table-striped table-hover\" summary=\"%s\">\n", id, summary);
 }
 
 /*! \brief End table in html output format.
@@ -633,17 +589,10 @@ static void table_end(FILE *restrict f)
  */
 static void newsection(FILE *restrict f, char const *restrict title)
 {
-	newrow(f);
-	output_line(f, "td", "lalign", "&nbsp;");
-	endrow(f);
-	newrow(f);
-	output_line(f, "th", "section", title);
-	endrow(f);
+	output_line(f, "h3", title);
 }
 
-/*! \brief Output html format.
- * FIXME: This function should return void.
- */
+/*! \brief Output html format. */
 int output_html(void)
 {
 	unsigned int i;
@@ -652,10 +601,11 @@ int output_html(void)
 	struct shared_network_t *shared_p;
 	int ret;
 	FILE *outfile;
+
 	if (config.output_file[0]) {
 		outfile = fopen(config.output_file, "w+");
 		if (outfile == NULL) {
-			err(EXIT_FAILURE, "output_html: %s", config.output_file);
+			error(EXIT_FAILURE, errno, "output_html: %s", config.output_file);
 		}
 	} else {
 		outfile = stdout;
@@ -664,38 +614,137 @@ int output_html(void)
 	range_p = ranges;
 	range_size = get_range_size(range_p);
 	shared_p = shared_networks;
-	if (config.fullhtml) {
-		html_header(outfile);
-	}
-	table_start(outfile);
-	if (config.output_limit[0] & BIT1) {
-		newsection(outfile, "Ranges:");
-		newrow(outfile);
-		output_line(outfile, "th", "lalign", "shared net name");
-		output_line(outfile, "th", "lalign", "first ip");
-		output_line(outfile, "th", "lalign", "last ip");
-		output_line(outfile, "th", "ralign", "max");
-		output_line(outfile, "th", "ralign", "cur");
-		output_line(outfile, "th", "ralign", "percent");
-		output_line(outfile, "th", "ralign", "touch");
-		output_line(outfile, "th", "ralign", "t+c");
-		output_line(outfile, "th", "ralign", "t+c perc");
-		if (config.backups_found == true) {
-			output_line(outfile, "th", "ralign", "bu");
-			output_line(outfile, "th", "ralign", "bu perc");
+	html_header(outfile);
+	newsection(outfile, "Sum of all");
+	table_start(outfile, "a", "all");
+	if (config.header_limit & A_BIT) {
+		start_tag(outfile, "thead");
+		start_tag(outfile, "tr");
+		output_line(outfile, "th", "name");
+		output_line(outfile, "th", "max");
+		output_line(outfile, "th", "cur");
+		output_line(outfile, "th", "percent");
+		output_line(outfile, "th", "touch");
+		output_line(outfile, "th", "t+c");
+		output_line(outfile, "th", "t+c perc");
+		if (config.backups_found == 1) {
+			output_line(outfile, "th", "bu");
+			output_line(outfile, "th", "bu perc");
 		}
-		endrow(outfile);
+		end_tag(outfile, "tr");
+		end_tag(outfile, "thead");
 	}
-	if (config.output_limit[1] & BIT1) {
-		for (i = 0; i < num_ranges; i++) {
-			newrow(outfile);
-			if (range_p->shared_net) {
-				output_line(outfile, "td", "lalign", range_p->shared_net->name);
-			} else {
-				output_line(outfile, "td", "lalign", "not_defined");
+	if (config.number_limit & A_BIT) {
+		start_tag(outfile, "tbody");
+		start_tag(outfile, "tr");
+		output_line(outfile, "td", shared_networks->name);
+		output_double(outfile, "td", shared_networks->available);
+		output_double(outfile, "td", shared_networks->used);
+		output_float(outfile, "td",
+			     shared_networks->available ==
+			     0 ? NAN : (float)(100 * shared_networks->used) /
+			     shared_networks->available);
+		output_double(outfile, "td", shared_networks->touched);
+		output_double(outfile, "td", shared_networks->touched + shared_networks->used);
+		output_float(outfile, "td",
+			     shared_networks->available == 0 ? NAN : (float)(100 *
+									      (shared_networks->touched
+									       +
+									       shared_networks->used))
+			     / shared_networks->available);
+		if (config.backups_found == 1) {
+			output_double(outfile, "td", shared_networks->backups);
+			output_float(outfile, "td",
+				     shared_networks->available == 0 ? NAN : (float)(100 *
+										      shared_networks->backups)
+				     / shared_networks->available);
+		}
+		end_tag(outfile, "tr");
+		end_tag(outfile, "tbody");
+	}
+	table_end(outfile);
+	newsection(outfile, "Shared networks");
+	table_start(outfile, "s", "snet");
+	if (config.header_limit & S_BIT) {
+		start_tag(outfile, "thead");
+		start_tag(outfile, "tr");
+		output_line(outfile, "th", "name");
+		output_line(outfile, "th", "max");
+		output_line(outfile, "th", "cur");
+		output_line(outfile, "th", "percent");
+		output_line(outfile, "th", "touch");
+		output_line(outfile, "th", "t+c");
+		output_line(outfile, "th", "t+c perc");
+		if (config.backups_found == 1) {
+			output_line(outfile, "th", "bu");
+			output_line(outfile, "th", "bu perc");
+		}
+		end_tag(outfile, "tr");
+		end_tag(outfile, "thead");
+	}
+	if (config.number_limit & S_BIT) {
+		start_tag(outfile, "tbody");
+		for (i = 0; i < num_shared_networks; i++) {
+			shared_p++;
+			start_tag(outfile, "tr");
+			output_line(outfile, "td", shared_p->name);
+			output_double(outfile, "td", shared_p->available);
+			output_double(outfile, "td", shared_p->used);
+			output_float(outfile, "td",
+				     shared_p->available ==
+				     0 ? NAN : (float)(100 * shared_p->used) /
+				     shared_p->available);
+			output_double(outfile, "td", shared_p->touched);
+			output_double(outfile, "td", shared_p->touched + shared_p->used);
+			output_float(outfile, "td",
+				     shared_p->available == 0 ? NAN : (float)(100 *
+									       (shared_p->touched +
+										shared_p->used)) /
+				     shared_p->available);
+			if (config.backups_found == 1) {
+				output_double(outfile, "td", shared_p->backups);
+				output_float(outfile, "td",
+					     shared_p->available == 0 ? NAN : (float)(100 *
+										       shared_p->backups)
+					     / shared_p->available);
 			}
-			output_line(outfile, "td", "lalign", ntop_ipaddr(&range_p->first_ip));
-			output_line(outfile, "td", "lalign", ntop_ipaddr(&range_p->last_ip));
+			end_tag(outfile, "tr");
+		}
+		end_tag(outfile, "tbody");
+	}
+	table_end(outfile);
+	newsection(outfile, "Ranges");
+	table_start(outfile, "r", "ranges");
+	if (config.header_limit & R_BIT) {
+		start_tag(outfile, "thead");
+		start_tag(outfile, "tr");
+		output_line(outfile, "th", "shared net name");
+		output_line(outfile, "th", "first ip");
+		output_line(outfile, "th", "last ip");
+		output_line(outfile, "th", "max");
+		output_line(outfile, "th", "cur");
+		output_line(outfile, "th", "percent");
+		output_line(outfile, "th", "touch");
+		output_line(outfile, "th", "t+c");
+		output_line(outfile, "th", "t+c perc");
+		if (config.backups_found == 1) {
+			output_line(outfile, "th", "bu");
+			output_line(outfile, "th", "bu perc");
+		}
+		end_tag(outfile, "tr");
+		end_tag(outfile, "thead");
+	}
+	if (config.number_limit & R_BIT) {
+		start_tag(outfile, "tbody");
+		for (i = 0; i < num_ranges; i++) {
+			start_tag(outfile, "tr");
+			if (range_p->shared_net) {
+				output_line(outfile, "td", range_p->shared_net->name);
+			} else {
+				output_line(outfile, "td", "not_defined");
+			}
+			output_line(outfile, "td", ntop_ipaddr(&range_p->first_ip));
+			output_line(outfile, "td", ntop_ipaddr(&range_p->last_ip));
 			output_double(outfile, "td", range_size);
 			output_double(outfile, "td", range_p->count);
 			output_float(outfile, "td", (float)(100 * range_p->count) / range_size);
@@ -704,130 +753,34 @@ int output_html(void)
 			output_float(outfile, "td",
 				     (float)(100 *
 					     (range_p->touched + range_p->count)) / range_size);
-			if (config.backups_found == true) {
+			if (config.backups_found == 1) {
 				output_double(outfile, "td", range_p->backups);
 				output_float(outfile, "td",
 					     (float)(100 * range_p->backups) / range_size);
 			}
-			endrow(outfile);
+			end_tag(outfile, "tr");
 			range_p++;
 			range_size = get_range_size(range_p);
 		}
+		end_tag(outfile, "tbody");
 	}
 	table_end(outfile);
-	table_start(outfile);
-	if (config.output_limit[0] & BIT2) {
-		newsection(outfile, "Shared networks:");
-		newrow(outfile);
-		output_line(outfile, "th", "lalign", "name");
-		output_line(outfile, "th", "ralign", "max");
-		output_line(outfile, "th", "ralign", "cur");
-		output_line(outfile, "th", "ralign", "percent");
-		output_line(outfile, "th", "ralign", "touch");
-		output_line(outfile, "th", "ralign", "t+c");
-		output_line(outfile, "th", "ralign", "t+c perc");
-		if (config.backups_found == true) {
-			output_line(outfile, "th", "ralign", "bu");
-			output_line(outfile, "th", "ralign", "bu perc");
-		}
-		endrow(outfile);
-	}
-	if (config.output_limit[1] & BIT2) {
-		for (i = 0; i < num_shared_networks; i++) {
-			shared_p++;
-			newrow(outfile);
-			output_line(outfile, "td", "lalign", shared_p->name);
-			output_double(outfile, "td", shared_p->available);
-			output_double(outfile, "td", shared_p->used);
-			output_float(outfile, "td",
-				     shared_p->available ==
-				     0 ? -NAN : (float)(100 * shared_p->used) /
-				     shared_p->available);
-			output_double(outfile, "td", shared_p->touched);
-			output_double(outfile, "td", shared_p->touched + shared_p->used);
-			output_float(outfile, "td",
-				     shared_p->available == 0 ? -NAN : (float)(100 *
-									       (shared_p->touched +
-										shared_p->used)) /
-				     shared_p->available);
-			if (config.backups_found == true) {
-				output_double(outfile, "td", shared_p->backups);
-				output_float(outfile, "td",
-					     shared_p->available == 0 ? -NAN : (float)(100 *
-										       shared_p->
-										       backups) /
-					     shared_p->available);
-			}
-
-			endrow(outfile);
-		}
-	}
-	if (config.output_limit[0] & BIT3) {
-		newsection(outfile, "Sum of all ranges:");
-		newrow(outfile);
-		output_line(outfile, "th", "lalign", "name");
-		output_line(outfile, "th", "ralign", "max");
-		output_line(outfile, "th", "ralign", "cur");
-		output_line(outfile, "th", "ralign", "percent");
-		output_line(outfile, "th", "ralign", "touch");
-		output_line(outfile, "th", "ralign", "t+c");
-		output_line(outfile, "th", "ralign", "t+c perc");
-		if (config.backups_found == true) {
-			output_line(outfile, "th", "ralign", "bu");
-			output_line(outfile, "th", "ralign", "bu perc");
-		}
-
-		endrow(outfile);
-	}
-	if (config.output_limit[1] & BIT3) {
-		newrow(outfile);
-		output_line(outfile, "td", "lalign", shared_networks->name);
-		output_double(outfile, "td", shared_networks->available);
-		output_double(outfile, "td", shared_networks->used);
-		output_float(outfile, "td",
-			     shared_networks->available ==
-			     0 ? -NAN : (float)(100 * shared_networks->used) /
-			     shared_networks->available);
-		output_double(outfile, "td", shared_networks->touched);
-		output_double(outfile, "td", shared_networks->touched + shared_networks->used);
-		output_float(outfile, "td",
-			     shared_networks->available == 0 ? -NAN : (float)(100 *
-									      (shared_networks->
-									       touched +
-									       shared_networks->
-									       used)) /
-			     shared_networks->available);
-		if (config.backups_found == true) {
-			output_double(outfile, "td", shared_networks->backups);
-			output_float(outfile, "td",
-				     shared_networks->available == 0 ? -NAN : (float)(100 *
-										      shared_networks->
-										      backups) /
-				     shared_networks->available);
-		}
-		endrow(outfile);
-	}
-	table_end(outfile);
-	if (config.fullhtml) {
-		html_footer(outfile);
-	}
+	html_footer(outfile);
 	if (outfile == stdout) {
 		ret = fflush(stdout);
 		if (ret) {
-			warn("output_html: fflush");
+			error(EXIT_FAILURE, errno, "output_html: fflush");
 		}
 	} else {
 		ret = close_stream(outfile);
 		if (ret) {
-			warn("output_html: fclose");
+			error(EXIT_FAILURE, errno, "output_html: fclose");
 		}
 	}
 	return 0;
 }
 
-/*! \brief Output cvs format.
- * FIXME: This function should return void.
- */
+/*! \brief Output cvs format. */
 int output_csv(void)
 {
 	unsigned int i;
@@ -836,10 +789,11 @@ int output_csv(void)
 	struct shared_network_t *shared_p;
 	FILE *outfile;
 	int ret;
+
 	if (config.output_file[0]) {
 		outfile = fopen(config.output_file, "w+");
 		if (outfile == NULL) {
-			err(EXIT_FAILURE, "output_csv: %s", config.output_file);
+			error(EXIT_FAILURE, errno, "output_csv: %s", config.output_file);
 		}
 	} else {
 		outfile = stdout;
@@ -848,17 +802,17 @@ int output_csv(void)
 	range_p = ranges;
 	range_size = get_range_size(range_p);
 	shared_p = shared_networks;
-	if (config.output_limit[0] & BIT1) {
+	if (config.header_limit & R_BIT) {
 		fprintf(outfile, "\"Ranges:\"\n");
 		fprintf
 		    (outfile,
 		     "\"shared net name\",\"first ip\",\"last ip\",\"max\",\"cur\",\"percent\",\"touch\",\"t+c\",\"t+c perc\"");
-		if (config.backups_found == true) {
+		if (config.backups_found == 1) {
 			fprintf(outfile, ",\"bu\",\"bu perc\"");
 		}
 		fprintf(outfile, "\n");
 	}
-	if (config.output_limit[1] & BIT1) {
+	if (config.number_limit & R_BIT) {
 		for (i = 0; i < num_ranges; i++) {
 			if (range_p->shared_net) {
 				fprintf(outfile, "\"%s\",", range_p->shared_net->name);
@@ -874,7 +828,7 @@ int output_csv(void)
 				range_p->touched,
 				range_p->touched + range_p->count,
 				(float)(100 * (range_p->touched + range_p->count)) / range_size);
-			if (config.backups_found == true) {
+			if (config.backups_found == 1) {
 				fprintf(outfile, ",\"%g\",\"%.3f\"",
 					range_p->backups,
 					(float)(100 * range_p->backups) / range_size);
@@ -886,16 +840,16 @@ int output_csv(void)
 		}
 		fprintf(outfile, "\n");
 	}
-	if (config.output_limit[0] & BIT2) {
+	if (config.header_limit & S_BIT) {
 		fprintf(outfile, "\"Shared networks:\"\n");
 		fprintf(outfile,
 			"\"name\",\"max\",\"cur\",\"percent\",\"touch\",\"t+c\",\"t+c perc\"");
-		if (config.backups_found == true) {
+		if (config.backups_found == 1) {
 			fprintf(outfile, ",\"bu\",\"bu perc\"");
 		}
 		fprintf(outfile, "\n");
 	}
-	if (config.output_limit[1] & BIT2) {
+	if (config.number_limit & S_BIT) {
 
 		for (i = 0; i < num_shared_networks; i++) {
 			shared_p++;
@@ -903,18 +857,18 @@ int output_csv(void)
 				"\"%s\",\"%g\",\"%g\",\"%.3f\",\"%g\",\"%g\",\"%.3f\"",
 				shared_p->name, shared_p->available,
 				shared_p->used,
-				shared_p->available == 0 ? -NAN : (float)(100 * shared_p->used) /
+				shared_p->available == 0 ? NAN : (float)(100 * shared_p->used) /
 				shared_p->available, shared_p->touched,
 				shared_p->touched + shared_p->used,
-				shared_p->available == 0 ? -NAN : (float)(100 *
+				shared_p->available == 0 ? NAN : (float)(100 *
 									  (shared_p->touched +
 									   shared_p->used)) /
 				shared_p->available);
-			if (config.backups_found == true) {
+			if (config.backups_found == 1) {
 				fprintf(outfile, ",\"%g\",\"%.3f\"",
 					shared_p->backups,
 					shared_p->available ==
-					0 ? -NAN : (float)(100 * shared_p->backups) /
+					0 ? NAN : (float)(100 * shared_p->backups) /
 					shared_p->available);
 			}
 
@@ -922,34 +876,34 @@ int output_csv(void)
 		}
 		fprintf(outfile, "\n");
 	}
-	if (config.output_limit[0] & BIT3) {
+	if (config.header_limit & A_BIT) {
 		fprintf(outfile, "\"Sum of all ranges:\"\n");
 		fprintf(outfile,
 			"\"name\",\"max\",\"cur\",\"percent\",\"touch\",\"t+c\",\"t+c perc\"");
-		if (config.backups_found == true) {
+		if (config.backups_found == 1) {
 			fprintf(outfile, ",\"bu\",\"bu perc\"");
 		}
 		fprintf(outfile, "\n");
 	}
-	if (config.output_limit[1] & BIT3) {
+	if (config.number_limit & A_BIT) {
 
 		fprintf(outfile,
 			"\"%s\",\"%g\",\"%g\",\"%.3f\",\"%g\",\"%g\",\"%.3f\"",
 			shared_networks->name, shared_networks->available,
 			shared_networks->used,
 			shared_networks->available ==
-			0 ? -NAN : (float)(100 * shared_networks->used) /
+			0 ? NAN : (float)(100 * shared_networks->used) /
 			shared_networks->available, shared_networks->touched,
 			shared_networks->touched + shared_networks->used,
 			shared_networks->available ==
-			0 ? -NAN : (float)(100 *
+			0 ? NAN : (float)(100 *
 					   (shared_networks->touched +
 					    shared_networks->used)) / shared_networks->available);
-		if (config.backups_found == true) {
+		if (config.backups_found == 1) {
 			fprintf(outfile, "%7g %8.3f",
 				shared_networks->backups,
 				shared_networks->available ==
-				0 ? -NAN : (float)(100 * shared_networks->backups) /
+				0 ? NAN : (float)(100 * shared_networks->backups) /
 				shared_networks->available);
 		}
 		fprintf(outfile, "\n");
@@ -957,20 +911,18 @@ int output_csv(void)
 	if (outfile == stdout) {
 		ret = fflush(stdout);
 		if (ret) {
-			warn("output_cvs: fflush");
+			error(EXIT_FAILURE, errno, "output_cvs: fflush");
 		}
 	} else {
 		ret = close_stream(outfile);
 		if (ret) {
-			warn("output_cvs: fclose");
+			error(EXIT_FAILURE, errno, "output_cvs: fclose");
 		}
 	}
 	return 0;
 }
 
-/*! \brief Output alarm text, and return program exit value.
- * FIXME: This function should return void.
- */
+/*! \brief Output alarm text, and return program exit value. */
 int output_alarming(void)
 {
 	FILE *outfile;
@@ -989,19 +941,22 @@ int output_alarming(void)
 	if (config.output_file[0]) {
 		outfile = fopen(config.output_file, "w+");
 		if (outfile == NULL) {
-			err(EXIT_FAILURE, "output_alarming: %s", config.output_file);
+			error(EXIT_FAILURE, errno, "output_alarming: %s", config.output_file);
 		}
 	} else {
 		outfile = stdout;
 	}
 
-	if (config.output_limit[1] & BIT1) {
+	if (config.number_limit & R_BIT) {
 		for (i = 0; i < num_ranges; i++) {
+			if (config.snet_alarms && range_p->shared_net != shared_networks) {
+				continue;
+			}
 			if (config.minsize < range_size) {
 				perc = (float)(100 * range_p->count) / range_size;
-				if (config.critical < perc)
+				if (config.critical < perc && (range_size - range_p->count) < config.crit_count)
 					rc++;
-				else if (config.warning < perc)
+				else if (config.warning < perc && (range_size - range_p->count) < config.warn_count)
 					rw++;
 				else
 					ro++;
@@ -1012,16 +967,16 @@ int output_alarming(void)
 			range_size = get_range_size(range_p);
 		}
 	}
-	if (config.output_limit[1] & BIT2) {
+	if (config.number_limit & S_BIT) {
 		for (i = 0; i < num_shared_networks; i++) {
 			shared_p++;
 			if (config.minsize < shared_p->available) {
 				perc =
 				    shared_p->available ==
 				    0 ? 100 : (float)(100 * shared_p->used) / shared_p->available;
-				if (config.critical < perc)
+				if (config.critical < perc && shared_p->used < config.crit_count)
 					sc++;
-				else if (config.warning < perc)
+				else if (config.warning < perc && shared_p->used < config.warn_count)
 					sw++;
 				else
 					so++;
@@ -1038,19 +993,19 @@ int output_alarming(void)
 	else
 		ret_val = STATE_OK;
 
-	if ((0 < rc && config.output_limit[1] & BIT1)
-	    || (0 < sc && config.output_limit[1] & BIT2)) {
+	if ((0 < rc && config.number_limit & R_BIT)
+	    || (0 < sc && config.number_limit & S_BIT)) {
 		fprintf(outfile, "CRITICAL: %s:", program_name);
-	} else if ((0 < rw && config.output_limit[1] & BIT1)
-		   || (0 < sw && config.output_limit[1] & BIT2)) {
+	} else if ((0 < rw && config.number_limit & R_BIT)
+		   || (0 < sw && config.number_limit & S_BIT)) {
 		fprintf(outfile, "WARNING: %s:", program_name);
 	} else {
-		if (config.output_limit[1] & BIT3)
+		if (config.number_limit & A_BIT)
 			fprintf(outfile, "OK:");
 		else
 			return ret_val;
 	}
-	if (config.output_limit[0] & BIT1) {
+	if (config.header_limit & R_BIT) {
 		fprintf(outfile, " Ranges - crit: %d warn: %d ok: %d", rc, rw, ro);
 		if (ri != 0) {
 			fprintf(outfile, " ignored: %d", ri);
@@ -1063,7 +1018,7 @@ int output_alarming(void)
 	} else {
 		fprintf(outfile, " ");
 	}
-	if (config.output_limit[0] & BIT2) {
+	if (config.header_limit & S_BIT) {
 		fprintf(outfile, "Shared nets - crit: %d warn: %d ok: %d", sc, sw, so);
 		if (si != 0) {
 			fprintf(outfile, " ignored: %d", si);
@@ -1077,12 +1032,12 @@ int output_alarming(void)
 	if (outfile == stdout) {
 		ret = fflush(stdout);
 		if (ret) {
-			warn("output_alarming: fflush");
+			error(EXIT_FAILURE, errno, "output_alarming: fflush");
 		}
 	} else {
 		ret = close_stream(outfile);
 		if (ret) {
-			warn("output_alarming: fclose");
+			error(EXIT_FAILURE, errno, "output_alarming: fclose");
 		}
 	}
 	return ret_val;
